/*
===============================================================================
   ___________.___ _______  ________    _________._________________________
   \_   _____/|   |\      \ \______ \  /   _____/|   \__    ___/\_   _____/
    |    __)  |   |/   |   \ |    |  \ \_____  \ |   | |    |    |    __)_
    |     \   |   /    |    \|    `   \/        \|   | |    |    |        \
    \___  /   |___\____|__  /_______  /_______  /|___| |____|   /_______  /
        \/                \/        \/        \/                        \/
                                          __        __
                          ____ ___  ___  / /_____ _/ /
                         / __ `__ \/ _ \/ __/ __ `/ / 
                        / / / / / /  __/ /_/ /_/ / /  
                       /_/ /_/ /_/\___/\__/\__,_/_/   

   FINDSITE-METAL predicts metal-binding sites, residues and metal preferences
   from template structures identified by threading

   Make sure you have libf2c, gzstream, zlib and libsvm installed

   This software is distributed WITHOUT ANY WARRANTY

   Please report bugs to michal@gatech.edu

   Center for the Study of Systems Biology
   250 14th Street NW, Atlanta, GA 30318, USA
===============================================================================
*/


#include "cluster.h"

using namespace std;

int cluster_avelink( double csim1[][MAXSIM], int csim2[], int csim3, double csim4, std::string csim5 )
{
 list< list<int> > clusters;
 
 for ( int ci1 = 0; ci1 < csim3; ci1++ )
 {
  list<int> cluster (1, ci1);
  
  clusters.push_back( cluster );
  
  csim2[ci1] = 0;
 }
 
 int cnum1 = clusters.size();
 
 bool cw1 = true;
 
 while ( cw1 && cnum1 > 1 )
 {
  list< list<int> >::iterator ci2;
  list< list<int> >::iterator ci3;
  
  int ci4, ci5;
  
  double cmin1 = 1000000.0;
  
  if ( csim5 == "max" )
   cmin1 = -1000000.0;
  
  list< list<int> >::iterator cmin2;
  list< list<int> >::iterator cmin3;
  
  for ( ci2 = clusters.begin(), ci4 = 0; ci2 != clusters.end(); ci2++, ci4++ )
  {
   for ( ci3 = clusters.begin(), ci5 = 0; ci3 != clusters.end(); ci3++, ci5++ )
   {
    if ( ci4 < cnum1 - 1 && ci5 > ci4 )
    {
     double amin1 = 0.0;
     double amin2 = 0.0;
     
     list<int>::iterator ci6;
     list<int>::iterator ci7;
     
     if ( !(*ci2).empty() && !(*ci3).empty() )
     {
      for ( ci6 = (*ci2).begin(); ci6 != (*ci2).end(); ci6++ )
       for ( ci7 = (*ci3).begin(); ci7 != (*ci3).end(); ci7++ )
       {
        if ( *ci6 < *ci7 )
         amin1 += csim1[(*ci6)][(*ci7)];
        else
         amin1 += csim1[(*ci7)][(*ci6)];
        
        amin2 += 1.0;
       }
       
      if ( amin2 > 0.1 )
       amin1 /= amin2;
      
      if ( ( csim5 == "min" && amin1 < cmin1 ) || ( csim5 == "max" && amin1 > cmin1 ) )
      {
       cmin1 = amin1;
       cmin2 = ci2;
       cmin3 = ci3;
      }
     }
    }
   }
  }
  
  if ( ( csim5 == "min" && cmin1 <= csim4 ) || ( csim5 == "max" && cmin1 >= csim4 ) )
  {
   (*cmin2).merge(*cmin3);
   
   (*cmin3).clear();
   
   cnum1--;
  }
  else
  {
   cw1 = false;
  }
 }
 
 multimap<int, list<int> > clusters_sorted;
 
 list< list<int> >::iterator ci8;
 
 for ( ci8 = clusters.begin(); ci8 != clusters.end(); ci8++ )
  if ( !(*ci8).empty() )
   clusters_sorted.insert( pair<int, list<int> >((*ci8).size(),*ci8) );
 
 multimap<int, list<int> >::iterator ci9;
 
 int cnum2 = cnum1;
 
 for ( ci9=clusters_sorted.begin() ; ci9 != clusters_sorted.end(); ci9++ )
 {
  cnum2--;
  
  for ( list<int>::iterator ci10 = ((*ci9).second).begin(); ci10 != ((*ci9).second).end(); ci10++)
   csim2[*ci10] = cnum2;
 }
 
 return cnum1;
}

int cluster_avelink_metal( double csim1[][MAXMET], int csim2[], int csim3, double csim4, std::string csim5 )
{
 list< list<int> > clusters;
 
 for ( int ci1 = 0; ci1 < csim3; ci1++ )
 {
  list<int> cluster (1, ci1);
  
  clusters.push_back( cluster );
  
  csim2[ci1] = 0;
 }
 
 int cnum1 = clusters.size();
 
 bool cw1 = true;
 
 while ( cw1 && cnum1 > 1 )
 {
  list< list<int> >::iterator ci2;
  list< list<int> >::iterator ci3;
  
  int ci4, ci5;
  
  double cmin1 = 1000000.0;
  
  if ( csim5 == "max" )
   cmin1 = -1000000.0;
  
  list< list<int> >::iterator cmin2;
  list< list<int> >::iterator cmin3;
  
  for ( ci2 = clusters.begin(), ci4 = 0; ci2 != clusters.end(); ci2++, ci4++ )
  {
   for ( ci3 = clusters.begin(), ci5 = 0; ci3 != clusters.end(); ci3++, ci5++ )
   {
    if ( ci4 < cnum1 - 1 && ci5 > ci4 )
    {
     double amin1 = 0.0;
     double amin2 = 0.0;
     
     list<int>::iterator ci6;
     list<int>::iterator ci7;
     
     if ( !(*ci2).empty() && !(*ci3).empty() )
     {
      for ( ci6 = (*ci2).begin(); ci6 != (*ci2).end(); ci6++ )
       for ( ci7 = (*ci3).begin(); ci7 != (*ci3).end(); ci7++ )
       {
        if ( *ci6 < *ci7 )
         amin1 += csim1[(*ci6)][(*ci7)];
        else
         amin1 += csim1[(*ci7)][(*ci6)];
        
        amin2 += 1.0;
       }
       
      if ( amin2 > 0.1 )
       amin1 /= amin2;
      
      if ( ( csim5 == "min" && amin1 < cmin1 ) || ( csim5 == "max" && amin1 > cmin1 ) )
      {
       cmin1 = amin1;
       cmin2 = ci2;
       cmin3 = ci3;
      }
     }
    }
   }
  }
  
  if ( ( csim5 == "min" && cmin1 <= csim4 ) || ( csim5 == "max" && cmin1 >= csim4 ) )
  {
   (*cmin2).merge(*cmin3);
   
   (*cmin3).clear();
   
   cnum1--;
  }
  else
  {
   cw1 = false;
  }
 }
 
 multimap<int, list<int> > clusters_sorted;
 
 list< list<int> >::iterator ci8;
 
 for ( ci8 = clusters.begin(); ci8 != clusters.end(); ci8++ )
  if ( !(*ci8).empty() )
   clusters_sorted.insert( pair<int, list<int> >((*ci8).size(),*ci8) );
 
 multimap<int, list<int> >::iterator ci9;
 
 int cnum2 = cnum1;
 
 for ( ci9=clusters_sorted.begin() ; ci9 != clusters_sorted.end(); ci9++ )
 {
  cnum2--;
  
  for ( list<int>::iterator ci10 = ((*ci9).second).begin(); ci10 != ((*ci9).second).end(); ci10++)
   csim2[*ci10] = cnum2;
 }
 
 return cnum1;
}
