/*
===============================================================================
   ___________.___ _______  ________    _________._________________________
   \_   _____/|   |\      \ \______ \  /   _____/|   \__    ___/\_   _____/
    |    __)  |   |/   |   \ |    |  \ \_____  \ |   | |    |    |    __)_
    |     \   |   /    |    \|    `   \/        \|   | |    |    |        \
    \___  /   |___\____|__  /_______  /_______  /|___| |____|   /_______  /
        \/                \/        \/        \/                        \/
                                          __        __
                          ____ ___  ___  / /_____ _/ /
                         / __ `__ \/ _ \/ __/ __ `/ / 
                        / / / / / /  __/ /_/ /_/ / /  
                       /_/ /_/ /_/\___/\__/\__,_/_/   

   FINDSITE-METAL predicts metal-binding sites, residues and metal preferences
   from template structures identified by threading

   Make sure you have libf2c, gzstream, zlib and libsvm installed

   This software is distributed WITHOUT ANY WARRANTY

   Please report bugs to michal@gatech.edu

   Center for the Study of Systems Biology
   250 14th Street NW, Atlanta, GA 30318, USA
===============================================================================
*/


#include "coords.h"

using namespace std;

// ==================================================================================   CoordsProtein

CoordsProtein::CoordsProtein( int an, int ar, double ax, double ay, double az, string at, string aa )
{
 n = an;
 r = ar;
 x = ax;
 y = ay;
 z = az;
 t = at;
 a = aa;
}

CoordsProtein::CoordsProtein( void )
{
 n = 0;
 r = 0;
 x = 0.0;
 y = 0.0;
 z = 0.0;
 t = "";
 a = "";
}

CoordsProtein::~CoordsProtein() {}

int CoordsProtein::getAtomNumber( void )
{
 return n;
}

int CoordsProtein::getResidueNumber( void )
{
 return r;
}

double CoordsProtein::getCoords( int an )
{
 switch (an)
 {
  case  1  :  return x;
  case  2  :  return y;
  case  3  :  return z;

  default  :  return 0;
 }
}

void CoordsProtein::setCoords( double ax, double ay, double az)
{
 x = ax;
 y = ay;
 z = az;
}

string CoordsProtein::getResidueName( void )
{
 return t;
}

string CoordsProtein::getAtomName( void )
{
 return a;
}
