/*
===============================================================================
   ___________.___ _______  ________    _________._________________________
   \_   _____/|   |\      \ \______ \  /   _____/|   \__    ___/\_   _____/
    |    __)  |   |/   |   \ |    |  \ \_____  \ |   | |    |    |    __)_
    |     \   |   /    |    \|    `   \/        \|   | |    |    |        \
    \___  /   |___\____|__  /_______  /_______  /|___| |____|   /_______  /
        \/                \/        \/        \/                        \/
                                          __        __
                          ____ ___  ___  / /_____ _/ /
                         / __ `__ \/ _ \/ __/ __ `/ / 
                        / / / / / /  __/ /_/ /_/ / /  
                       /_/ /_/ /_/\___/\__/\__,_/_/   

   FINDSITE-METAL predicts metal-binding sites, residues and metal preferences
   from template structures identified by threading

   Make sure you have libf2c, gzstream, zlib and libsvm installed

   This software is distributed WITHOUT ANY WARRANTY

   Please report bugs to michal@gatech.edu

   Center for the Study of Systems Biology
   250 14th Street NW, Atlanta, GA 30318, USA
===============================================================================
*/


#include "data.h"

using namespace std;

double getBkgFreq1( string resnam1 )
{
      if ( resnam1 == "ALA" || resnam1 == "A" ) { return 0.0813; }
 else if ( resnam1 == "CYS" || resnam1 == "C" ) { return 0.0142; }
 else if ( resnam1 == "ASP" || resnam1 == "D" ) { return 0.0540; }
 else if ( resnam1 == "GLU" || resnam1 == "E" ) { return 0.0673; }
 else if ( resnam1 == "PHE" || resnam1 == "F" ) { return 0.0388; }
 else if ( resnam1 == "GLY" || resnam1 == "G" ) { return 0.0704; }
 else if ( resnam1 == "HIS" || resnam1 == "H" ) { return 0.0228; }
 else if ( resnam1 == "ILE" || resnam1 == "I" ) { return 0.0592; }
 else if ( resnam1 == "LYS" || resnam1 == "K" ) { return 0.0588; }
 else if ( resnam1 == "LEU" || resnam1 == "L" ) { return 0.0967; }
 else if ( resnam1 == "MET" || resnam1 == "M" ) { return 0.0241; }
 else if ( resnam1 == "ASN" || resnam1 == "N" ) { return 0.0405; }
 else if ( resnam1 == "PRO" || resnam1 == "P" ) { return 0.0477; }
 else if ( resnam1 == "GLN" || resnam1 == "Q" ) { return 0.0395; }
 else if ( resnam1 == "ARG" || resnam1 == "R" ) { return 0.0550; }
 else if ( resnam1 == "SER" || resnam1 == "S" ) { return 0.0667; }
 else if ( resnam1 == "THR" || resnam1 == "T" ) { return 0.0535; }
 else if ( resnam1 == "VAL" || resnam1 == "V" ) { return 0.0682; }
 else if ( resnam1 == "TRP" || resnam1 == "W" ) { return 0.0109; }
 else if ( resnam1 == "TYR" || resnam1 == "Y" ) { return 0.0293; }
 
 else
 {
  cout << "Unknown residue passed to getBackFreq1: " << resnam1 << endl << endl;
  exit(EXIT_FAILURE);
 }
 
 exit(EXIT_FAILURE);
}

double getBkgFreq2( int metnum1 )
{
      if ( metnum1 == 0 ) { return 0.255444; }
 else if ( metnum1 == 1 ) { return 0.033501; }
 else if ( metnum1 == 2 ) { return 0.040201; }
 else if ( metnum1 == 3 ) { return 0.126466; }
 else if ( metnum1 == 4 ) { return 0.142379; }
 else if ( metnum1 == 5 ) { return 0.096315; }
 else if ( metnum1 == 6 ) { return 0.019263; }
 else if ( metnum1 == 7 ) { return 0.286432; }
 
 else
 {
  cout << "Unknown metal passed to getBackFreq2: " << metnum1 << endl << endl;
  exit(EXIT_FAILURE);
 }
 
 exit(EXIT_FAILURE);
}

double getBkgFreq3( int metnum1, int resnum1 )
{
      if ( metnum1 == 0 )
 {
       if ( resnum1 ==  0 ) { return 0.027273; }
  else if ( resnum1 ==  1 ) { return 0.003636; }
  else if ( resnum1 ==  2 ) { return 0.393636; }
  else if ( resnum1 ==  3 ) { return 0.164545; }
  else if ( resnum1 ==  4 ) { return 0.010909; }
  else if ( resnum1 ==  5 ) { return 0.063636; }
  else if ( resnum1 ==  6 ) { return 0.015455; }
  else if ( resnum1 ==  7 ) { return 0.020909; }
  else if ( resnum1 ==  8 ) { return 0.019091; }
  else if ( resnum1 ==  9 ) { return 0.016364; }
  else if ( resnum1 == 10 ) { return 0.005455; }
  else if ( resnum1 == 11 ) { return 0.093636; }
  else if ( resnum1 == 12 ) { return 0.009091; }
  else if ( resnum1 == 13 ) { return 0.014545; }
  else if ( resnum1 == 14 ) { return 0.010000; }
  else if ( resnum1 == 15 ) { return 0.038182; }
  else if ( resnum1 == 16 ) { return 0.042727; }
  else if ( resnum1 == 17 ) { return 0.027273; }
  else if ( resnum1 == 18 ) { return 0.006364; }
  else if ( resnum1 == 19 ) { return 0.017273; }
 }
 else if ( metnum1 == 1 )
 {
       if ( resnum1 ==  0 ) { return 0.000001; }
  else if ( resnum1 ==  1 ) { return 0.120968; }
  else if ( resnum1 ==  2 ) { return 0.250000; }
  else if ( resnum1 ==  3 ) { return 0.169355; }
  else if ( resnum1 ==  4 ) { return 0.000001; }
  else if ( resnum1 ==  5 ) { return 0.008065; }
  else if ( resnum1 ==  6 ) { return 0.387097; }
  else if ( resnum1 ==  7 ) { return 0.000001; }
  else if ( resnum1 ==  8 ) { return 0.000001; }
  else if ( resnum1 ==  9 ) { return 0.000001; }
  else if ( resnum1 == 10 ) { return 0.024194; }
  else if ( resnum1 == 11 ) { return 0.008065; }
  else if ( resnum1 == 12 ) { return 0.000001; }
  else if ( resnum1 == 13 ) { return 0.008065; }
  else if ( resnum1 == 14 ) { return 0.000001; }
  else if ( resnum1 == 15 ) { return 0.016129; }
  else if ( resnum1 == 16 ) { return 0.000001; }
  else if ( resnum1 == 17 ) { return 0.000001; }
  else if ( resnum1 == 18 ) { return 0.000001; }
  else if ( resnum1 == 19 ) { return 0.008065; }
 }
 else if ( metnum1 == 2 )
 {
       if ( resnum1 ==  0 ) { return 0.000001; }
  else if ( resnum1 ==  1 ) { return 0.182927; }
  else if ( resnum1 ==  2 ) { return 0.012195; }
  else if ( resnum1 ==  3 ) { return 0.024390; }
  else if ( resnum1 ==  4 ) { return 0.000001; }
  else if ( resnum1 ==  5 ) { return 0.006098; }
  else if ( resnum1 ==  6 ) { return 0.634146; }
  else if ( resnum1 ==  7 ) { return 0.000001; }
  else if ( resnum1 ==  8 ) { return 0.000001; }
  else if ( resnum1 ==  9 ) { return 0.000001; }
  else if ( resnum1 == 10 ) { return 0.103659; }
  else if ( resnum1 == 11 ) { return 0.000001; }
  else if ( resnum1 == 12 ) { return 0.006098; }
  else if ( resnum1 == 13 ) { return 0.006098; }
  else if ( resnum1 == 14 ) { return 0.000001; }
  else if ( resnum1 == 15 ) { return 0.006098; }
  else if ( resnum1 == 16 ) { return 0.006098; }
  else if ( resnum1 == 17 ) { return 0.000001; }
  else if ( resnum1 == 18 ) { return 0.000001; }
  else if ( resnum1 == 19 ) { return 0.012195; }
 }
 else if ( metnum1 == 3 )
 {
       if ( resnum1 ==  0 ) { return 0.000001; }
  else if ( resnum1 ==  1 ) { return 0.166667; }
  else if ( resnum1 ==  2 ) { return 0.058201; }
  else if ( resnum1 ==  3 ) { return 0.100529; }
  else if ( resnum1 ==  4 ) { return 0.000001; }
  else if ( resnum1 ==  5 ) { return 0.000001; }
  else if ( resnum1 ==  6 ) { return 0.579365; }
  else if ( resnum1 ==  7 ) { return 0.000001; }
  else if ( resnum1 ==  8 ) { return 0.002646; }
  else if ( resnum1 ==  9 ) { return 0.000001; }
  else if ( resnum1 == 10 ) { return 0.055556; }
  else if ( resnum1 == 11 ) { return 0.002646; }
  else if ( resnum1 == 12 ) { return 0.000001; }
  else if ( resnum1 == 13 ) { return 0.000001; }
  else if ( resnum1 == 14 ) { return 0.000001; }
  else if ( resnum1 == 15 ) { return 0.002646; }
  else if ( resnum1 == 16 ) { return 0.000001; }
  else if ( resnum1 == 17 ) { return 0.000001; }
  else if ( resnum1 == 18 ) { return 0.000001; }
  else if ( resnum1 == 19 ) { return 0.031746; }
 }
 else if ( metnum1 == 4 )
 {
       if ( resnum1 ==  0 ) { return 0.030516; }
  else if ( resnum1 ==  1 ) { return 0.002347; }
  else if ( resnum1 ==  2 ) { return 0.375587; }
  else if ( resnum1 ==  3 ) { return 0.199531; }
  else if ( resnum1 ==  4 ) { return 0.007042; }
  else if ( resnum1 ==  5 ) { return 0.021127; }
  else if ( resnum1 ==  6 ) { return 0.021127; }
  else if ( resnum1 ==  7 ) { return 0.014085; }
  else if ( resnum1 ==  8 ) { return 0.023474; }
  else if ( resnum1 ==  9 ) { return 0.018779; }
  else if ( resnum1 == 10 ) { return 0.004695; }
  else if ( resnum1 == 11 ) { return 0.065728; }
  else if ( resnum1 == 12 ) { return 0.002347; }
  else if ( resnum1 == 13 ) { return 0.023474; }
  else if ( resnum1 == 14 ) { return 0.025822; }
  else if ( resnum1 == 15 ) { return 0.063380; }
  else if ( resnum1 == 16 ) { return 0.056338; }
  else if ( resnum1 == 17 ) { return 0.023474; }
  else if ( resnum1 == 18 ) { return 0.007042; }
  else if ( resnum1 == 19 ) { return 0.014085; }
 }
 else if ( metnum1 == 5 )
 {
       if ( resnum1 ==  0 ) { return 0.002841; }
  else if ( resnum1 ==  1 ) { return 0.002841; }
  else if ( resnum1 ==  2 ) { return 0.431818; }
  else if ( resnum1 ==  3 ) { return 0.201705; }
  else if ( resnum1 ==  4 ) { return 0.008523; }
  else if ( resnum1 ==  5 ) { return 0.002841; }
  else if ( resnum1 ==  6 ) { return 0.210227; }
  else if ( resnum1 ==  7 ) { return 0.014205; }
  else if ( resnum1 ==  8 ) { return 0.005682; }
  else if ( resnum1 ==  9 ) { return 0.002841; }
  else if ( resnum1 == 10 ) { return 0.005682; }
  else if ( resnum1 == 11 ) { return 0.042614; }
  else if ( resnum1 == 12 ) { return 0.002841; }
  else if ( resnum1 == 13 ) { return 0.008523; }
  else if ( resnum1 == 14 ) { return 0.005682; }
  else if ( resnum1 == 15 ) { return 0.028409; }
  else if ( resnum1 == 16 ) { return 0.019886; }
  else if ( resnum1 == 17 ) { return 0.000001; }
  else if ( resnum1 == 18 ) { return 0.000001; }
  else if ( resnum1 == 19 ) { return 0.002841; }
 }
 else if ( metnum1 == 6 )
 {
       if ( resnum1 ==  0 ) { return 0.014286; }
  else if ( resnum1 ==  1 ) { return 0.100000; }
  else if ( resnum1 ==  2 ) { return 0.171429; }
  else if ( resnum1 ==  3 ) { return 0.100000; }
  else if ( resnum1 ==  4 ) { return 0.000001; }
  else if ( resnum1 ==  5 ) { return 0.028571; }
  else if ( resnum1 ==  6 ) { return 0.514286; }
  else if ( resnum1 ==  7 ) { return 0.000001; }
  else if ( resnum1 ==  8 ) { return 0.000001; }
  else if ( resnum1 ==  9 ) { return 0.000001; }
  else if ( resnum1 == 10 ) { return 0.028571; }
  else if ( resnum1 == 11 ) { return 0.014286; }
  else if ( resnum1 == 12 ) { return 0.000001; }
  else if ( resnum1 == 13 ) { return 0.000001; }
  else if ( resnum1 == 14 ) { return 0.000001; }
  else if ( resnum1 == 15 ) { return 0.014286; }
  else if ( resnum1 == 16 ) { return 0.000001; }
  else if ( resnum1 == 17 ) { return 0.000001; }
  else if ( resnum1 == 18 ) { return 0.000001; }
  else if ( resnum1 == 19 ) { return 0.014286; }
 }
 else if ( metnum1 == 7 )
 {
       if ( resnum1 ==  0 ) { return 0.000001; }
  else if ( resnum1 ==  1 ) { return 0.413916; }
  else if ( resnum1 ==  2 ) { return 0.123104; }
  else if ( resnum1 ==  3 ) { return 0.089206; }
  else if ( resnum1 ==  4 ) { return 0.000001; }
  else if ( resnum1 ==  5 ) { return 0.000892; }
  else if ( resnum1 ==  6 ) { return 0.339875; }
  else if ( resnum1 ==  7 ) { return 0.001784; }
  else if ( resnum1 ==  8 ) { return 0.002676; }
  else if ( resnum1 ==  9 ) { return 0.000892; }
  else if ( resnum1 == 10 ) { return 0.003568; }
  else if ( resnum1 == 11 ) { return 0.002676; }
  else if ( resnum1 == 12 ) { return 0.000892; }
  else if ( resnum1 == 13 ) { return 0.005352; }
  else if ( resnum1 == 14 ) { return 0.001784; }
  else if ( resnum1 == 15 ) { return 0.004460; }
  else if ( resnum1 == 16 ) { return 0.002676; }
  else if ( resnum1 == 17 ) { return 0.000001; }
  else if ( resnum1 == 18 ) { return 0.003568; }
  else if ( resnum1 == 19 ) { return 0.002676; }
 }
 
 exit(EXIT_FAILURE);
}

std::string three2oneS( std::string resnam1 )
{
      if ( resnam1 == "ALA" ) { return "A"; }
 else if ( resnam1 == "CYS" ) { return "C"; }
 else if ( resnam1 == "ASP" ) { return "D"; }
 else if ( resnam1 == "GLU" ) { return "E"; }
 else if ( resnam1 == "PHE" ) { return "F"; }
 else if ( resnam1 == "GLY" ) { return "G"; }
 else if ( resnam1 == "HIS" ) { return "H"; }
 else if ( resnam1 == "ILE" ) { return "I"; }
 else if ( resnam1 == "LYS" ) { return "K"; }
 else if ( resnam1 == "LEU" ) { return "L"; }
 else if ( resnam1 == "MET" ) { return "M"; }
 else if ( resnam1 == "ASN" ) { return "N"; }
 else if ( resnam1 == "PRO" ) { return "P"; }
 else if ( resnam1 == "GLN" ) { return "Q"; }
 else if ( resnam1 == "ARG" ) { return "R"; }
 else if ( resnam1 == "SER" ) { return "S"; }
 else if ( resnam1 == "THR" ) { return "T"; }
 else if ( resnam1 == "VAL" ) { return "V"; }
 else if ( resnam1 == "TRP" ) { return "W"; }
 else if ( resnam1 == "TYR" ) { return "Y"; }
 
 else
 {
  cout << "Unknown residue passed to three2one: " << resnam1 << endl << endl;
  exit(EXIT_FAILURE);
 }
 
 exit(EXIT_FAILURE);
}

char three2oneC( std::string resnam1 )
{
      if ( resnam1 == "ALA" ) { return 'A'; }
 else if ( resnam1 == "CYS" ) { return 'C'; }
 else if ( resnam1 == "ASP" ) { return 'D'; }
 else if ( resnam1 == "GLU" ) { return 'E'; }
 else if ( resnam1 == "PHE" ) { return 'F'; }
 else if ( resnam1 == "GLY" ) { return 'G'; }
 else if ( resnam1 == "HIS" ) { return 'H'; }
 else if ( resnam1 == "ILE" ) { return 'I'; }
 else if ( resnam1 == "LYS" ) { return 'K'; }
 else if ( resnam1 == "LEU" ) { return 'L'; }
 else if ( resnam1 == "MET" ) { return 'M'; }
 else if ( resnam1 == "ASN" ) { return 'N'; }
 else if ( resnam1 == "PRO" ) { return 'P'; }
 else if ( resnam1 == "GLN" ) { return 'Q'; }
 else if ( resnam1 == "ARG" ) { return 'R'; }
 else if ( resnam1 == "SER" ) { return 'S'; }
 else if ( resnam1 == "THR" ) { return 'T'; }
 else if ( resnam1 == "VAL" ) { return 'V'; }
 else if ( resnam1 == "TRP" ) { return 'W'; }
 else if ( resnam1 == "TYR" ) { return 'Y'; }
 
 else
 {
  cout << "Unknown residue passed to three2one: " << resnam1 << endl << endl;
  exit(EXIT_FAILURE);
 }
 
 exit(EXIT_FAILURE);
}

std::string one2three( std::string resnam1 )
{
      if ( resnam1 == "A" ) { return "ALA"; }
 else if ( resnam1 == "C" ) { return "CYS"; }
 else if ( resnam1 == "D" ) { return "ASP"; }
 else if ( resnam1 == "E" ) { return "GLU"; }
 else if ( resnam1 == "F" ) { return "PHE"; }
 else if ( resnam1 == "G" ) { return "GLY"; }
 else if ( resnam1 == "H" ) { return "HIS"; }
 else if ( resnam1 == "I" ) { return "ILE"; }
 else if ( resnam1 == "K" ) { return "LYS"; }
 else if ( resnam1 == "L" ) { return "LEU"; }
 else if ( resnam1 == "M" ) { return "MET"; }
 else if ( resnam1 == "N" ) { return "ASN"; }
 else if ( resnam1 == "P" ) { return "PRO"; }
 else if ( resnam1 == "Q" ) { return "GLN"; }
 else if ( resnam1 == "R" ) { return "ARG"; }
 else if ( resnam1 == "S" ) { return "SER"; }
 else if ( resnam1 == "T" ) { return "THR"; }
 else if ( resnam1 == "V" ) { return "VAL"; }
 else if ( resnam1 == "W" ) { return "TRP"; }
 else if ( resnam1 == "Y" ) { return "TYR"; }
 
 else
 {
  cout << "Unknown residue passed to one2three: " << resnam1 << endl << endl;
  exit(EXIT_FAILURE);
 }
 
 exit(EXIT_FAILURE);
}

int two2num( std::string metnam1 )
{
      if ( metnam1 == "CA" ) { return 0; }
 else if ( metnam1 == "CO" ) { return 1; }
 else if ( metnam1 == "CU" ) { return 2; }
 else if ( metnam1 == "FE" ) { return 3; }
 else if ( metnam1 == "MG" ) { return 4; }
 else if ( metnam1 == "MN" ) { return 5; }
 else if ( metnam1 == "NI" ) { return 6; }
 else if ( metnam1 == "ZN" ) { return 7; }
 
 else
 {
  cout << "Unknown metal passed to two2num: " << metnam1 << endl << endl;
  exit(EXIT_FAILURE);
 }
 
 exit(EXIT_FAILURE);
}

std::string num2two( int metnum1 )
{
      if ( metnum1 == 0 ) { return "CA"; }
 else if ( metnum1 == 1 ) { return "CO"; }
 else if ( metnum1 == 2 ) { return "CU"; }
 else if ( metnum1 == 3 ) { return "FE"; }
 else if ( metnum1 == 4 ) { return "MG"; }
 else if ( metnum1 == 5 ) { return "MN"; }
 else if ( metnum1 == 6 ) { return "NI"; }
 else if ( metnum1 == 7 ) { return "ZN"; }
 
 else
 {
  cout << "Unknown residue passed to num2two: " << metnum1 << endl << endl;
  exit(EXIT_FAILURE);
 }
 
 exit(EXIT_FAILURE);
}

int one2num( char resnam1 )
{
      if ( resnam1 == 'A' ) { return  0; }
 else if ( resnam1 == 'C' ) { return  1; }
 else if ( resnam1 == 'D' ) { return  2; }
 else if ( resnam1 == 'E' ) { return  3; }
 else if ( resnam1 == 'F' ) { return  4; }
 else if ( resnam1 == 'G' ) { return  5; }
 else if ( resnam1 == 'H' ) { return  6; }
 else if ( resnam1 == 'I' ) { return  7; }
 else if ( resnam1 == 'K' ) { return  8; }
 else if ( resnam1 == 'L' ) { return  9; }
 else if ( resnam1 == 'M' ) { return 10; }
 else if ( resnam1 == 'N' ) { return 11; }
 else if ( resnam1 == 'P' ) { return 12; }
 else if ( resnam1 == 'Q' ) { return 13; }
 else if ( resnam1 == 'R' ) { return 14; }
 else if ( resnam1 == 'S' ) { return 15; }
 else if ( resnam1 == 'T' ) { return 16; }
 else if ( resnam1 == 'V' ) { return 17; }
 else if ( resnam1 == 'W' ) { return 18; }
 else if ( resnam1 == 'Y' ) { return 19; }
 
 else
 {
  cout << "Unknown residue passed to one2num: " << resnam1 << endl << endl;
  exit(EXIT_FAILURE);
 }
 
 exit(EXIT_FAILURE);
}

std::string num2one( int resnum1 )
{
      if ( resnum1 ==  0 ) { return "A"; }
 else if ( resnum1 ==  1 ) { return "C"; }
 else if ( resnum1 ==  2 ) { return "D"; }
 else if ( resnum1 ==  3 ) { return "E"; }
 else if ( resnum1 ==  4 ) { return "F"; }
 else if ( resnum1 ==  5 ) { return "G"; }
 else if ( resnum1 ==  6 ) { return "H"; }
 else if ( resnum1 ==  7 ) { return "I"; }
 else if ( resnum1 ==  8 ) { return "K"; }
 else if ( resnum1 ==  9 ) { return "L"; }
 else if ( resnum1 == 10 ) { return "M"; }
 else if ( resnum1 == 11 ) { return "N"; }
 else if ( resnum1 == 12 ) { return "P"; }
 else if ( resnum1 == 13 ) { return "Q"; }
 else if ( resnum1 == 14 ) { return "R"; }
 else if ( resnum1 == 15 ) { return "S"; }
 else if ( resnum1 == 16 ) { return "T"; }
 else if ( resnum1 == 17 ) { return "V"; }
 else if ( resnum1 == 18 ) { return "W"; }
 else if ( resnum1 == 19 ) { return "Y"; }
 
 else
 {
  cout << "Unknown residue passed to num2one: " << resnum1 << endl << endl;
  exit(EXIT_FAILURE);
 }
 
 exit(EXIT_FAILURE);
}
