/*
===============================================================================
   ___________.___ _______  ________    _________._________________________
   \_   _____/|   |\      \ \______ \  /   _____/|   \__    ___/\_   _____/
    |    __)  |   |/   |   \ |    |  \ \_____  \ |   | |    |    |    __)_
    |     \   |   /    |    \|    `   \/        \|   | |    |    |        \
    \___  /   |___\____|__  /_______  /_______  /|___| |____|   /_______  /
        \/                \/        \/        \/                        \/
                                          __        __
                          ____ ___  ___  / /_____ _/ /
                         / __ `__ \/ _ \/ __/ __ `/ / 
                        / / / / / /  __/ /_/ /_/ / /  
                       /_/ /_/ /_/\___/\__/\__,_/_/   

   FINDSITE-METAL predicts metal-binding sites, residues and metal preferences
   from template structures identified by threading

   Make sure you have libf2c, gzstream, zlib and libsvm installed

   This software is distributed WITHOUT ANY WARRANTY

   Please report bugs to michal@gatech.edu

   Center for the Study of Systems Biology
   250 14th Street NW, Atlanta, GA 30318, USA
===============================================================================
*/


#include "list.h"

using namespace std;

void getList( string templates_name, string lib_path, string cluster_def, list<string> &complex_list1 )
{
 list<string> template_list;
 
 string line1;
 
 ifstream templates_file( templates_name.c_str() );
 
 if ( !templates_file.is_open() )  { cout << "Cannot open " << templates_name << endl; exit(EXIT_FAILURE); }
 
 while (getline(templates_file,line1))
  template_list.push_back( line1 );
 
 templates_file.close();
 
 template_list.sort();
 template_list.unique();
 
 multimap<string,string> cluster_list;
 
 string line2;
 
 ifstream clusters_file( cluster_def.c_str() );
 
 if ( !clusters_file.is_open() )  { cout << "Cannot open " << cluster_def << endl; exit(EXIT_FAILURE); }
 
 while (getline(clusters_file,line2))
 {
  string tpl1;
  int    tpl3;
  
  tpl1 = line2.substr(0,15);
  
  for ( unsigned int tpl6 = 0; tpl6 < tpl1.length(); tpl6++ )
   if ( tpl1[tpl6] == ' ' )
   {
    tpl1.erase(tpl6, 1);
    
    tpl6--;
   }
  
  tpl3 = atoi(line2.substr(15,5).c_str());
  
  for ( int tpl4 = 0; tpl4 < tpl3; tpl4++ )
  {
   string tpl5;
   
   tpl5 = line2.substr(21+6*tpl4,5);
   
   cluster_list.insert( pair<string,string>(tpl1,tpl5) );
  }
 }
 
 templates_file.close();
 
 list<string> protein_list;
 
 list<string>::iterator i2;
 
 for ( i2 = template_list.begin(); i2 != template_list.end(); i2++ )
 {
  multimap<string,string>::iterator i3;
  pair<multimap<string,string>::iterator,multimap<string,string>::iterator> i4;
  
  i4 = cluster_list.equal_range(*i2);
  
  for ( i3 = i4.first; i3 != i4.second; i3++ )
   protein_list.push_back( (*i3).second );
 }
 
 protein_list.sort();
 protein_list.unique();
 
 set<string> p_list;
 set<string> m_list;
 
 string line3, line4;
 
 string p_name = lib_path+"/PROTEINS.LST";
 string m_name = lib_path+"/METALS.LST";
 
 ifstream p_file( p_name.c_str() );
 ifstream m_file( m_name.c_str() );
 
 if ( !p_file.is_open() )  { cout << "Cannot open " << p_name << endl; exit(EXIT_FAILURE); }
 
 while (getline(p_file,line3))
  p_list.insert(line3);
 
 p_file.close();
 
 if ( !m_file.is_open() )  { cout << "Cannot open " << m_name << endl; exit(EXIT_FAILURE); }
 
 while (getline(m_file,line4))
  m_list.insert(line4);
 
 m_file.close();
 
 list<string>::iterator i5;
 
 for ( i5 = protein_list.begin(); i5 != protein_list.end(); i5++ )
  if ( p_list.count(*i5) > 0 && m_list.count(*i5) > 0 )
   complex_list1.push_back(*i5);
 
 cout << "FINDSITE library: " << lib_path <<  endl
      << "FINDSITE mapping: " << cluster_def << endl << endl
 
      << "Total number of proteins in the library:             " << setw(9) << p_list.size() << endl
      << "Total number of metal-bound proteins in the library: " << setw(9) << m_list.size() << endl << endl
 
      << "Number of metal-bound templates: " << setw(5) << complex_list1.size() << endl;
}
