/*
===============================================================================
   ___________.___ _______  ________    _________._________________________
   \_   _____/|   |\      \ \______ \  /   _____/|   \__    ___/\_   _____/
    |    __)  |   |/   |   \ |    |  \ \_____  \ |   | |    |    |    __)_
    |     \   |   /    |    \|    `   \/        \|   | |    |    |        \
    \___  /   |___\____|__  /_______  /_______  /|___| |____|   /_______  /
        \/                \/        \/        \/                        \/
                                          __        __
                          ____ ___  ___  / /_____ _/ /
                         / __ `__ \/ _ \/ __/ __ `/ / 
                        / / / / / /  __/ /_/ /_/ / /  
                       /_/ /_/ /_/\___/\__/\__,_/_/   

   FINDSITE-METAL predicts metal-binding sites, residues and metal preferences
   from template structures identified by threading

   Make sure you have libf2c, gzstream, zlib and libsvm installed

   This software is distributed WITHOUT ANY WARRANTY

   Please report bugs to michal@gatech.edu

   Center for the Study of Systems Biology
   250 14th Street NW, Atlanta, GA 30318, USA
===============================================================================
*/


#include "runsvm.h"

using namespace std;

bool SVMscaling( std::string s1_name, double s2[][2] )
{
 string line1, line2;
 
 ifstream s1_file( s1_name.c_str() );
 
 if ( !s1_file.is_open() )
  return true;
 
 int i1 = 0;
 
 while (getline(s1_file,line1))
  if ( i1++ > 1 )
  {
   istringstream iss(line1, istringstream::in);
   
   int i2 = 0;
   int i3 = 0;
   double i4 = 0.0;
   double i5 = 0.0;
   
   while( iss >> line2 )     
   {
         if ( i2 == 0 ) { i3 = atoi(line2.c_str()) - 1; }
    else if ( i2 == 1 ) { i4 = atof(line2.c_str()); }
    else if ( i2 == 2 ) { i5 = atof(line2.c_str()); }
    
    i2++;
   }
   
   s2[i3][0] = i4;
   s2[i3][1] = i5;
  }
 
 s1_file.close();
 
 return false;
}

void SVMnormalize( double & v, double lb, double ub )
{
 v = ( ( v - lb ) / ( ub - lb ) ) * 2.0 - 1.0;
 
 if ( v < -1.0 )
  v = -1.0;
 if ( v > 1.0 )
  v = 1.0;
}

double SVMpredict( struct svm_model * & model1, struct svm_node * & node1, double value1[] )
{
 int nr_class = svm_get_nr_class(model1);
 double *prob_estimates = NULL;
 
 int *labels = ( int * ) malloc( nr_class * sizeof( int ) );
 
 svm_get_labels(model1,labels);
 
 prob_estimates = ( double * ) malloc( nr_class * sizeof( double ) );
 
 int r1 = 0;
 
 if ( labels[1] )
  r1 = 1;
 
 free(labels);
 
 for ( int i = 0; i < MAXSVM; i++ )
 {
  node1[i].index  = i + 1;
  node1[i].value  = value1[i];
 }
 
 node1[MAXSVM].index = -1;
 
 double p1 = 0.0;
 
 p1 = svm_predict_probability(model1,node1,prob_estimates);
 
 double p2 = prob_estimates[r1];
 
 free(prob_estimates);
 
 return p2;
}
